<?php

global $g_comp_database;

// Set migrationId
$migrationIdentifier = "major_update_migration";

// Check whether migration was executed before
if ($this->is_migration_done($migrationIdentifier)) {
    $g_migration_log[] = '<span class="bold">1.19 update has been done already.</span>';
} else {
    $g_migration_log[] = '<span class="bold">Processing some necessary 1.19 updates.</span>';

    $dao = \isys_cmdb_dao::instance($g_comp_database);

    $reportModuleId = (int)$dao
        ->retrieve("SELECT isys_module__id FROM isys_module WHERE isys_module__const = 'C__MODULE__REPORT' LIMIT 1;")
        ->get_row_value('isys_module__id');

    $personGroupAdminObjectId = (int)$dao
        ->retrieve("SELECT isys_obj__id FROM isys_obj WHERE isys_obj__const = 'C__OBJ__PERSON_GROUP_ADMIN' LIMIT 1;")
        ->get_row_value('isys_obj__id');

    // @see ID-7000 Add rights, if necessary
    if ($reportModuleId && $personGroupAdminObjectId) {
        $g_migration_log[] = '<span class="bold">Check if rights need to be created.</span>';
        $conditionQuery = "SELECT COUNT(1) AS cnt
            FROM isys_auth 
            WHERE isys_auth__path = 'EXPORT_REPORT_CONFIG' 
            AND isys_auth__isys_obj__id = {$personGroupAdminObjectId}
            LIMIT 1;";

        if (!$dao->retrieve($conditionQuery)->get_row_value('cnt')) {
            $insertQuery = "INSERT INTO isys_auth SET
                isys_auth__isys_obj__id = {$personGroupAdminObjectId},
                isys_auth__type = 8,
                isys_auth__isys_module__id = {$reportModuleId},
                isys_auth__path = 'EXPORT_REPORT_CONFIG';";

            $dao->update($insertQuery) && $dao->apply_update();
            $g_migration_log[] = '<span>Allow admin group users to export reports.</span>';
        } else {
            $g_migration_log[] = '<span>Admin group is already allowed to export reports.</span>';
        }

        $conditionQuery = "SELECT COUNT(1) AS cnt 
            FROM isys_auth 
            WHERE isys_auth__path = 'IMPORT_REPORT_CONFIG'
            AND isys_auth__isys_obj__id = {$personGroupAdminObjectId}
            LIMIT 1;";

        if (!$dao->retrieve($conditionQuery)->get_row_value('cnt')) {
            $insertQuery = "INSERT INTO isys_auth SET
                    isys_auth__isys_obj__id = {$personGroupAdminObjectId},
                    isys_auth__type = 8,
                    isys_auth__isys_module__id = {$reportModuleId},
                    isys_auth__path = 'IMPORT_REPORT_CONFIG';";

            $dao->update($insertQuery) && $dao->apply_update();
            $g_migration_log[] = '<span>Allow admin group users to import reports.</span>';
        } else {
            $g_migration_log[] = '<span>Admin group is already allowed to import reports.</span>';
        }
    }

    $proModuleId = (int)$dao
        ->retrieve("SELECT isys_module__id FROM isys_module WHERE isys_module__const = 'C__MODULE__PRO' LIMIT 1;")
        ->get_row_value('isys_module__id');

    // @see ID-8349 Only add the 'pro' add-on entry if it does not exist AND if we actually use i-doit PRO
    $g_migration_log[] = '<span class="bold">Check if "PRO" is necessary.</span>';

    if (class_exists('\isys_module_pro') && !$proModuleId) {
        $insertQuery = "INSERT INTO isys_module SET
            isys_module__title = 'LC__MODULE__PRO',
            isys_module__identifier = 'pro',
            isys_module__const = 'C__MODULE__PRO',
            isys_module__date_install = NOW(),
            isys_module__class = 'isys_module_pro',
            isys_module__persistent = 0,
            isys_module__status = 2,
            isys_module__parent = NULL;";

        $dao->update($insertQuery) && $dao->apply_update();
        $g_migration_log[] = '<span>Create "PRO" entry for the new language features.</span>';
    } else {
        $g_migration_log[] = '<span>"PRO" entry is not necessary.</span>';
    }

    $g_migration_log[] = '<span class="bold">Migration finished!</span>';

    $this->migration_done($migrationIdentifier);
}
